/****************************************************************************
** Meta object code from reading C++ file 'parser.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../heaptrack-1.5.0/src/analyze/gui/parser.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'parser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6ParserE_t {};
} // unnamed namespace

template <> constexpr inline auto Parser::qt_create_metaobjectdata<qt_meta_tag_ZN6ParserE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Parser",
        "progressMessageAvailable",
        "",
        "progress",
        "summaryAvailable",
        "SummaryData",
        "summary",
        "bottomUpDataAvailable",
        "TreeData",
        "data",
        "topDownDataAvailable",
        "callerCalleeDataAvailable",
        "CallerCalleeResults",
        "consumedChartDataAvailable",
        "ChartData",
        "allocationsChartDataAvailable",
        "temporaryChartDataAvailable",
        "sizeHistogramDataAvailable",
        "HistogramData",
        "finished",
        "failedToOpen",
        "path"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'progressMessageAvailable'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(const int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'summaryAvailable'
        QtMocHelpers::SignalData<void(const SummaryData &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'bottomUpDataAvailable'
        QtMocHelpers::SignalData<void(const TreeData &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'topDownDataAvailable'
        QtMocHelpers::SignalData<void(const TreeData &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'callerCalleeDataAvailable'
        QtMocHelpers::SignalData<void(const CallerCalleeResults &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 9 },
        }}),
        // Signal 'consumedChartDataAvailable'
        QtMocHelpers::SignalData<void(const ChartData &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 9 },
        }}),
        // Signal 'allocationsChartDataAvailable'
        QtMocHelpers::SignalData<void(const ChartData &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 9 },
        }}),
        // Signal 'temporaryChartDataAvailable'
        QtMocHelpers::SignalData<void(const ChartData &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 9 },
        }}),
        // Signal 'sizeHistogramDataAvailable'
        QtMocHelpers::SignalData<void(const HistogramData &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 9 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'failedToOpen'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Parser, qt_meta_tag_ZN6ParserE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Parser::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6ParserE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6ParserE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6ParserE_t>.metaTypes,
    nullptr
} };

void Parser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Parser *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->progressMessageAvailable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->progress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->summaryAvailable((*reinterpret_cast<std::add_pointer_t<SummaryData>>(_a[1]))); break;
        case 3: _t->bottomUpDataAvailable((*reinterpret_cast<std::add_pointer_t<TreeData>>(_a[1]))); break;
        case 4: _t->topDownDataAvailable((*reinterpret_cast<std::add_pointer_t<TreeData>>(_a[1]))); break;
        case 5: _t->callerCalleeDataAvailable((*reinterpret_cast<std::add_pointer_t<CallerCalleeResults>>(_a[1]))); break;
        case 6: _t->consumedChartDataAvailable((*reinterpret_cast<std::add_pointer_t<ChartData>>(_a[1]))); break;
        case 7: _t->allocationsChartDataAvailable((*reinterpret_cast<std::add_pointer_t<ChartData>>(_a[1]))); break;
        case 8: _t->temporaryChartDataAvailable((*reinterpret_cast<std::add_pointer_t<ChartData>>(_a[1]))); break;
        case 9: _t->sizeHistogramDataAvailable((*reinterpret_cast<std::add_pointer_t<HistogramData>>(_a[1]))); break;
        case 10: _t->finished(); break;
        case 11: _t->failedToOpen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SummaryData >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TreeData >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TreeData >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CallerCalleeResults >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ChartData >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ChartData >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ChartData >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HistogramData >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const QString & )>(_a, &Parser::progressMessageAvailable, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const int )>(_a, &Parser::progress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const SummaryData & )>(_a, &Parser::summaryAvailable, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const TreeData & )>(_a, &Parser::bottomUpDataAvailable, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const TreeData & )>(_a, &Parser::topDownDataAvailable, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const CallerCalleeResults & )>(_a, &Parser::callerCalleeDataAvailable, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const ChartData & )>(_a, &Parser::consumedChartDataAvailable, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const ChartData & )>(_a, &Parser::allocationsChartDataAvailable, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const ChartData & )>(_a, &Parser::temporaryChartDataAvailable, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const HistogramData & )>(_a, &Parser::sizeHistogramDataAvailable, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)()>(_a, &Parser::finished, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(const QString & )>(_a, &Parser::failedToOpen, 11))
            return;
    }
}

const QMetaObject *Parser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Parser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6ParserE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Parser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void Parser::progressMessageAvailable(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Parser::progress(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Parser::summaryAvailable(const SummaryData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Parser::bottomUpDataAvailable(const TreeData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Parser::topDownDataAvailable(const TreeData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Parser::callerCalleeDataAvailable(const CallerCalleeResults & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Parser::consumedChartDataAvailable(const ChartData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Parser::allocationsChartDataAvailable(const ChartData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Parser::temporaryChartDataAvailable(const ChartData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Parser::sizeHistogramDataAvailable(const HistogramData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Parser::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Parser::failedToOpen(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
