/****************************************************************************
** Meta object code from reading C++ file 'callercalleemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../heaptrack-1.5.0/src/analyze/gui/callercalleemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'callercalleemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17CallerCalleeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CallerCalleeModel::qt_create_metaobjectdata<qt_meta_tag_ZN17CallerCalleeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CallerCalleeModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CallerCalleeModel, qt_meta_tag_ZN17CallerCalleeModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CallerCalleeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<HashModel<CallerCalleeEntryMap,CallerCalleeModel>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CallerCalleeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CallerCalleeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17CallerCalleeModelE_t>.metaTypes,
    nullptr
} };

void CallerCalleeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CallerCalleeModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CallerCalleeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CallerCalleeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CallerCalleeModelE_t>.strings))
        return static_cast<void*>(this);
    return HashModel<CallerCalleeEntryMap,CallerCalleeModel>::qt_metacast(_clname);
}

int CallerCalleeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HashModel<CallerCalleeEntryMap,CallerCalleeModel>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11CallerModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CallerModel::qt_create_metaobjectdata<qt_meta_tag_ZN11CallerModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CallerModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CallerModel, qt_meta_tag_ZN11CallerModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CallerModel::staticMetaObject = { {
    QMetaObject::SuperData::link<SymbolCostModelImpl<CallerModel>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CallerModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CallerModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CallerModelE_t>.metaTypes,
    nullptr
} };

void CallerModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CallerModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CallerModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CallerModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CallerModelE_t>.strings))
        return static_cast<void*>(this);
    return SymbolCostModelImpl<CallerModel>::qt_metacast(_clname);
}

int CallerModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SymbolCostModelImpl<CallerModel>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11CalleeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CalleeModel::qt_create_metaobjectdata<qt_meta_tag_ZN11CalleeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CalleeModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CalleeModel, qt_meta_tag_ZN11CalleeModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CalleeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<SymbolCostModelImpl<CalleeModel>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CalleeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CalleeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CalleeModelE_t>.metaTypes,
    nullptr
} };

void CalleeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CalleeModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CalleeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CalleeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CalleeModelE_t>.strings))
        return static_cast<void*>(this);
    return SymbolCostModelImpl<CalleeModel>::qt_metacast(_clname);
}

int CalleeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SymbolCostModelImpl<CalleeModel>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14SourceMapModelE_t {};
} // unnamed namespace

template <> constexpr inline auto SourceMapModel::qt_create_metaobjectdata<qt_meta_tag_ZN14SourceMapModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SourceMapModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SourceMapModel, qt_meta_tag_ZN14SourceMapModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SourceMapModel::staticMetaObject = { {
    QMetaObject::SuperData::link<LocationCostModelImpl<SourceMapModel>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SourceMapModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SourceMapModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14SourceMapModelE_t>.metaTypes,
    nullptr
} };

void SourceMapModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SourceMapModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SourceMapModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SourceMapModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SourceMapModelE_t>.strings))
        return static_cast<void*>(this);
    return LocationCostModelImpl<SourceMapModel>::qt_metacast(_clname);
}

int SourceMapModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LocationCostModelImpl<SourceMapModel>::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
