/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "flamegraph.h"
#include "kcollapsiblegroupbox.h"
#include "kmessagewidget.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *actionResetFilter;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout_6;
    KMessageWidget *messages;
    QStackedWidget *pages;
    QWidget *openPage;
    QVBoxLayout *verticalLayout_15;
    QSpacerItem *verticalSpacer_4;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    KUrlRequester *openFile;
    KUrlRequester *compareTo;
    QLabel *openFileLabel;
    QLabel *compareToLabel;
    QLabel *suppressionsLabel;
    KUrlRequester *suppressions;
    QDialogButtonBox *buttonBox;
    QSpacerItem *verticalSpacer_3;
    QWidget *loadingPage;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer;
    QLabel *loadingLabel;
    QProgressBar *loadingProgress;
    QLabel *progressLabel;
    QSpacerItem *verticalSpacer_2;
    QWidget *resultsPage;
    QVBoxLayout *verticalLayout_2;
    QTabWidget *tabWidget;
    QWidget *summaryTab;
    QVBoxLayout *verticalLayout_7;
    QWidget *widget_10;
    QHBoxLayout *horizontalLayout_6;
    QLabel *summaryLeft;
    QLabel *summaryCenter;
    QLabel *summaryRight;
    QFrame *line;
    QWidget *widget_4;
    QHBoxLayout *horizontalLayout_2;
    QSplitter *splitter;
    QWidget *widget_5;
    QVBoxLayout *verticalLayout_8;
    QLabel *topPeakLabel;
    QTreeView *topPeak;
    QWidget *widget_6;
    QVBoxLayout *verticalLayout_9;
    QLabel *topLeakedLabel;
    QTreeView *topLeaked;
    QWidget *widget_7;
    QVBoxLayout *verticalLayout_10;
    QLabel *topAllocationsLabel;
    QTreeView *topAllocations;
    QWidget *widget_8;
    QVBoxLayout *verticalLayout_11;
    QLabel *topTemporaryLabel;
    QTreeView *topTemporary;
    KCollapsibleGroupBox *suppressionBox;
    QHBoxLayout *horizontalLayout_8;
    QTreeView *suppressionsView;
    QWidget *bottomUpTab;
    QVBoxLayout *verticalLayout;
    QWidget *widget;
    QHBoxLayout *horizontalLayout;
    QLineEdit *bottomUpFilterFunction;
    QLineEdit *bottomUpFilterFile;
    QLineEdit *bottomUpFilterModule;
    QTreeView *bottomUpResults;
    QWidget *callerCalleeTab;
    QVBoxLayout *verticalLayout_13;
    QWidget *widget_11;
    QHBoxLayout *horizontalLayout_7;
    QLineEdit *callerCalleeFilterFunction;
    QLineEdit *callerCalleeFilterModule;
    QWidget *widget_12;
    QVBoxLayout *verticalLayout_12;
    QSplitter *splitter_2;
    QTreeView *callerCalleeResults;
    QWidget *widget_9;
    QHBoxLayout *horizontalLayout_3;
    QTreeView *callerView;
    QTreeView *calleeView;
    QTreeView *locationView;
    QWidget *topDownTab;
    QVBoxLayout *verticalLayout_4;
    QWidget *widget_2;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *topDownFilterFunction;
    QLineEdit *topDownFilterFile;
    QLineEdit *topDownFilterModule;
    QTreeView *topDownResults;
    FlameGraph *flameGraphTab;
    QDockWidget *stacksDock;
    QWidget *stacksDockContents;
    QVBoxLayout *verticalLayout_5;
    QWidget *widget_3;
    QHBoxLayout *horizontalLayout_5;
    QLabel *stackSpinnerLabel;
    QSpinBox *stackSpinner;
    QTreeView *stacksTree;
    QMenuBar *menuBar;
    QMenu *menu_File;
    QMenu *menu_Settings;
    QMenu *menuFilter;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(1332, 896);
        actionResetFilter = new QAction(MainWindow);
        actionResetFilter->setObjectName("actionResetFilter");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("timeline-use-zone-off")));
        actionResetFilter->setIcon(icon);
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        verticalLayout_6 = new QVBoxLayout(centralwidget);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        messages = new KMessageWidget(centralwidget);
        messages->setObjectName("messages");
        messages->setMessageType(KMessageWidget::Information);
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("dialog-error")));
        messages->setIcon(icon1);

        verticalLayout_6->addWidget(messages);

        pages = new QStackedWidget(centralwidget);
        pages->setObjectName("pages");
        openPage = new QWidget();
        openPage->setObjectName("openPage");
        verticalLayout_15 = new QVBoxLayout(openPage);
        verticalLayout_15->setObjectName("verticalLayout_15");
        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_15->addItem(verticalSpacer_4);

        groupBox = new QGroupBox(openPage);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        openFile = new KUrlRequester(groupBox);
        openFile->setObjectName("openFile");
        openFile->setFileDialogModality(Qt::WindowModal);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, openFile);

        compareTo = new KUrlRequester(groupBox);
        compareTo->setObjectName("compareTo");
        compareTo->setFileDialogModality(Qt::WindowModal);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, compareTo);

        openFileLabel = new QLabel(groupBox);
        openFileLabel->setObjectName("openFileLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, openFileLabel);

        compareToLabel = new QLabel(groupBox);
        compareToLabel->setObjectName("compareToLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, compareToLabel);

        suppressionsLabel = new QLabel(groupBox);
        suppressionsLabel->setObjectName("suppressionsLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, suppressionsLabel);

        suppressions = new KUrlRequester(groupBox);
        suppressions->setObjectName("suppressions");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, suppressions);


        verticalLayout_15->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(openPage);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setEnabled(false);
        buttonBox->setStandardButtons(QDialogButtonBox::Open);

        verticalLayout_15->addWidget(buttonBox);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_15->addItem(verticalSpacer_3);

        pages->addWidget(openPage);
        loadingPage = new QWidget();
        loadingPage->setObjectName("loadingPage");
        verticalLayout_3 = new QVBoxLayout(loadingPage);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        loadingLabel = new QLabel(loadingPage);
        loadingLabel->setObjectName("loadingLabel");
        loadingLabel->setText(QString::fromUtf8("Loading file, please wait..."));
        loadingLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_3->addWidget(loadingLabel);

        loadingProgress = new QProgressBar(loadingPage);
        loadingProgress->setObjectName("loadingProgress");
        loadingProgress->setValue(24);

        verticalLayout_3->addWidget(loadingProgress);

        progressLabel = new QLabel(loadingPage);
        progressLabel->setObjectName("progressLabel");
        progressLabel->setText(QString::fromUtf8("Progress Message..."));
        progressLabel->setAlignment(Qt::AlignCenter);
        progressLabel->setWordWrap(true);

        verticalLayout_3->addWidget(progressLabel);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        pages->addWidget(loadingPage);
        resultsPage = new QWidget();
        resultsPage->setObjectName("resultsPage");
        verticalLayout_2 = new QVBoxLayout(resultsPage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(resultsPage);
        tabWidget->setObjectName("tabWidget");
        summaryTab = new QWidget();
        summaryTab->setObjectName("summaryTab");
        verticalLayout_7 = new QVBoxLayout(summaryTab);
        verticalLayout_7->setObjectName("verticalLayout_7");
        widget_10 = new QWidget(summaryTab);
        widget_10->setObjectName("widget_10");
        horizontalLayout_6 = new QHBoxLayout(widget_10);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        summaryLeft = new QLabel(widget_10);
        summaryLeft->setObjectName("summaryLeft");
        summaryLeft->setText(QString::fromUtf8("summary"));
        summaryLeft->setWordWrap(true);
        summaryLeft->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout_6->addWidget(summaryLeft);

        summaryCenter = new QLabel(widget_10);
        summaryCenter->setObjectName("summaryCenter");
        summaryCenter->setText(QString::fromUtf8("summary"));
        summaryCenter->setWordWrap(true);
        summaryCenter->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout_6->addWidget(summaryCenter);

        summaryRight = new QLabel(widget_10);
        summaryRight->setObjectName("summaryRight");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(summaryRight->sizePolicy().hasHeightForWidth());
        summaryRight->setSizePolicy(sizePolicy);
        summaryRight->setText(QString::fromUtf8("summary goes here"));
        summaryRight->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        summaryRight->setWordWrap(true);
        summaryRight->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout_6->addWidget(summaryRight);


        verticalLayout_7->addWidget(widget_10);

        line = new QFrame(summaryTab);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_7->addWidget(line);

        widget_4 = new QWidget(summaryTab);
        widget_4->setObjectName("widget_4");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(widget_4->sizePolicy().hasHeightForWidth());
        widget_4->setSizePolicy(sizePolicy1);
        horizontalLayout_2 = new QHBoxLayout(widget_4);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, -1, 0, 0);
        splitter = new QSplitter(widget_4);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        widget_5 = new QWidget(splitter);
        widget_5->setObjectName("widget_5");
        verticalLayout_8 = new QVBoxLayout(widget_5);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, -1, -1, -1);
        topPeakLabel = new QLabel(widget_5);
        topPeakLabel->setObjectName("topPeakLabel");
        QFont font;
        font.setBold(true);
        topPeakLabel->setFont(font);
        topPeakLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_8->addWidget(topPeakLabel);

        topPeak = new QTreeView(widget_5);
        topPeak->setObjectName("topPeak");
        topPeak->setAlternatingRowColors(true);
        topPeak->setRootIsDecorated(false);
        topPeak->setUniformRowHeights(true);

        verticalLayout_8->addWidget(topPeak);

        splitter->addWidget(widget_5);
        widget_6 = new QWidget(splitter);
        widget_6->setObjectName("widget_6");
        verticalLayout_9 = new QVBoxLayout(widget_6);
        verticalLayout_9->setObjectName("verticalLayout_9");
        topLeakedLabel = new QLabel(widget_6);
        topLeakedLabel->setObjectName("topLeakedLabel");
        topLeakedLabel->setFont(font);
        topLeakedLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_9->addWidget(topLeakedLabel);

        topLeaked = new QTreeView(widget_6);
        topLeaked->setObjectName("topLeaked");
        topLeaked->setAlternatingRowColors(true);
        topLeaked->setRootIsDecorated(false);
        topLeaked->setUniformRowHeights(true);

        verticalLayout_9->addWidget(topLeaked);

        splitter->addWidget(widget_6);
        widget_7 = new QWidget(splitter);
        widget_7->setObjectName("widget_7");
        verticalLayout_10 = new QVBoxLayout(widget_7);
        verticalLayout_10->setObjectName("verticalLayout_10");
        topAllocationsLabel = new QLabel(widget_7);
        topAllocationsLabel->setObjectName("topAllocationsLabel");
        topAllocationsLabel->setFont(font);
        topAllocationsLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_10->addWidget(topAllocationsLabel);

        topAllocations = new QTreeView(widget_7);
        topAllocations->setObjectName("topAllocations");
        topAllocations->setAlternatingRowColors(true);
        topAllocations->setRootIsDecorated(false);
        topAllocations->setUniformRowHeights(true);

        verticalLayout_10->addWidget(topAllocations);

        splitter->addWidget(widget_7);
        widget_8 = new QWidget(splitter);
        widget_8->setObjectName("widget_8");
        verticalLayout_11 = new QVBoxLayout(widget_8);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_11->setContentsMargins(-1, -1, 0, -1);
        topTemporaryLabel = new QLabel(widget_8);
        topTemporaryLabel->setObjectName("topTemporaryLabel");
        topTemporaryLabel->setFont(font);
        topTemporaryLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_11->addWidget(topTemporaryLabel);

        topTemporary = new QTreeView(widget_8);
        topTemporary->setObjectName("topTemporary");
        topTemporary->setAlternatingRowColors(true);
        topTemporary->setRootIsDecorated(false);
        topTemporary->setUniformRowHeights(true);

        verticalLayout_11->addWidget(topTemporary);

        splitter->addWidget(widget_8);

        horizontalLayout_2->addWidget(splitter);


        verticalLayout_7->addWidget(widget_4);

        suppressionBox = new KCollapsibleGroupBox(summaryTab);
        suppressionBox->setObjectName("suppressionBox");
        horizontalLayout_8 = new QHBoxLayout(suppressionBox);
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalLayout_8->setContentsMargins(0, 0, 0, 0);
        suppressionsView = new QTreeView(suppressionBox);
        suppressionsView->setObjectName("suppressionsView");
        suppressionsView->setAlternatingRowColors(true);
        suppressionsView->setRootIsDecorated(false);

        horizontalLayout_8->addWidget(suppressionsView);


        verticalLayout_7->addWidget(suppressionBox);

        tabWidget->addTab(summaryTab, QString());
        bottomUpTab = new QWidget();
        bottomUpTab->setObjectName("bottomUpTab");
        verticalLayout = new QVBoxLayout(bottomUpTab);
        verticalLayout->setObjectName("verticalLayout");
        widget = new QWidget(bottomUpTab);
        widget->setObjectName("widget");
        horizontalLayout = new QHBoxLayout(widget);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        bottomUpFilterFunction = new QLineEdit(widget);
        bottomUpFilterFunction->setObjectName("bottomUpFilterFunction");

        horizontalLayout->addWidget(bottomUpFilterFunction);

        bottomUpFilterFile = new QLineEdit(widget);
        bottomUpFilterFile->setObjectName("bottomUpFilterFile");

        horizontalLayout->addWidget(bottomUpFilterFile);

        bottomUpFilterModule = new QLineEdit(widget);
        bottomUpFilterModule->setObjectName("bottomUpFilterModule");

        horizontalLayout->addWidget(bottomUpFilterModule);


        verticalLayout->addWidget(widget);

        bottomUpResults = new QTreeView(bottomUpTab);
        bottomUpResults->setObjectName("bottomUpResults");
        bottomUpResults->setAlternatingRowColors(true);
        bottomUpResults->setIndentation(10);
        bottomUpResults->setRootIsDecorated(true);
        bottomUpResults->setUniformRowHeights(true);
        bottomUpResults->setSortingEnabled(true);

        verticalLayout->addWidget(bottomUpResults);

        tabWidget->addTab(bottomUpTab, QString());
        callerCalleeTab = new QWidget();
        callerCalleeTab->setObjectName("callerCalleeTab");
        verticalLayout_13 = new QVBoxLayout(callerCalleeTab);
        verticalLayout_13->setObjectName("verticalLayout_13");
        widget_11 = new QWidget(callerCalleeTab);
        widget_11->setObjectName("widget_11");
        horizontalLayout_7 = new QHBoxLayout(widget_11);
        horizontalLayout_7->setSpacing(6);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalLayout_7->setContentsMargins(0, 0, 0, 0);
        callerCalleeFilterFunction = new QLineEdit(widget_11);
        callerCalleeFilterFunction->setObjectName("callerCalleeFilterFunction");

        horizontalLayout_7->addWidget(callerCalleeFilterFunction);

        callerCalleeFilterModule = new QLineEdit(widget_11);
        callerCalleeFilterModule->setObjectName("callerCalleeFilterModule");

        horizontalLayout_7->addWidget(callerCalleeFilterModule);


        verticalLayout_13->addWidget(widget_11);

        widget_12 = new QWidget(callerCalleeTab);
        widget_12->setObjectName("widget_12");
        verticalLayout_12 = new QVBoxLayout(widget_12);
        verticalLayout_12->setObjectName("verticalLayout_12");
        verticalLayout_12->setContentsMargins(0, 0, 0, 0);
        splitter_2 = new QSplitter(widget_12);
        splitter_2->setObjectName("splitter_2");
        splitter_2->setOrientation(Qt::Vertical);
        callerCalleeResults = new QTreeView(splitter_2);
        callerCalleeResults->setObjectName("callerCalleeResults");
        callerCalleeResults->setAlternatingRowColors(true);
        callerCalleeResults->setRootIsDecorated(false);
        callerCalleeResults->setUniformRowHeights(true);
        callerCalleeResults->setSortingEnabled(true);
        splitter_2->addWidget(callerCalleeResults);
        widget_9 = new QWidget(splitter_2);
        widget_9->setObjectName("widget_9");
        horizontalLayout_3 = new QHBoxLayout(widget_9);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        callerView = new QTreeView(widget_9);
        callerView->setObjectName("callerView");
        callerView->setAlternatingRowColors(true);
        callerView->setRootIsDecorated(false);
        callerView->setSortingEnabled(true);

        horizontalLayout_3->addWidget(callerView);

        calleeView = new QTreeView(widget_9);
        calleeView->setObjectName("calleeView");
        calleeView->setAlternatingRowColors(true);
        calleeView->setRootIsDecorated(false);
        calleeView->setSortingEnabled(true);

        horizontalLayout_3->addWidget(calleeView);

        locationView = new QTreeView(widget_9);
        locationView->setObjectName("locationView");
        locationView->setAlternatingRowColors(true);
        locationView->setRootIsDecorated(false);
        locationView->setSortingEnabled(true);

        horizontalLayout_3->addWidget(locationView);

        splitter_2->addWidget(widget_9);

        verticalLayout_12->addWidget(splitter_2);


        verticalLayout_13->addWidget(widget_12);

        tabWidget->addTab(callerCalleeTab, QString());
        topDownTab = new QWidget();
        topDownTab->setObjectName("topDownTab");
        verticalLayout_4 = new QVBoxLayout(topDownTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        widget_2 = new QWidget(topDownTab);
        widget_2->setObjectName("widget_2");
        horizontalLayout_4 = new QHBoxLayout(widget_2);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        topDownFilterFunction = new QLineEdit(widget_2);
        topDownFilterFunction->setObjectName("topDownFilterFunction");

        horizontalLayout_4->addWidget(topDownFilterFunction);

        topDownFilterFile = new QLineEdit(widget_2);
        topDownFilterFile->setObjectName("topDownFilterFile");

        horizontalLayout_4->addWidget(topDownFilterFile);

        topDownFilterModule = new QLineEdit(widget_2);
        topDownFilterModule->setObjectName("topDownFilterModule");

        horizontalLayout_4->addWidget(topDownFilterModule);


        verticalLayout_4->addWidget(widget_2);

        topDownResults = new QTreeView(topDownTab);
        topDownResults->setObjectName("topDownResults");
        topDownResults->setAlternatingRowColors(true);
        topDownResults->setIndentation(10);
        topDownResults->setRootIsDecorated(true);
        topDownResults->setUniformRowHeights(true);
        topDownResults->setSortingEnabled(true);

        verticalLayout_4->addWidget(topDownResults);

        tabWidget->addTab(topDownTab, QString());
        flameGraphTab = new FlameGraph();
        flameGraphTab->setObjectName("flameGraphTab");
        tabWidget->addTab(flameGraphTab, QString());

        verticalLayout_2->addWidget(tabWidget);

        pages->addWidget(resultsPage);

        verticalLayout_6->addWidget(pages);

        MainWindow->setCentralWidget(centralwidget);
        stacksDock = new QDockWidget(MainWindow);
        stacksDock->setObjectName("stacksDock");
        stacksDock->setFeatures(QDockWidget::NoDockWidgetFeatures);
        stacksDockContents = new QWidget();
        stacksDockContents->setObjectName("stacksDockContents");
        verticalLayout_5 = new QVBoxLayout(stacksDockContents);
        verticalLayout_5->setObjectName("verticalLayout_5");
        widget_3 = new QWidget(stacksDockContents);
        widget_3->setObjectName("widget_3");
        horizontalLayout_5 = new QHBoxLayout(widget_3);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(0, 0, 0, 0);
        stackSpinnerLabel = new QLabel(widget_3);
        stackSpinnerLabel->setObjectName("stackSpinnerLabel");

        horizontalLayout_5->addWidget(stackSpinnerLabel);

        stackSpinner = new QSpinBox(widget_3);
        stackSpinner->setObjectName("stackSpinner");

        horizontalLayout_5->addWidget(stackSpinner);


        verticalLayout_5->addWidget(widget_3);

        stacksTree = new QTreeView(stacksDockContents);
        stacksTree->setObjectName("stacksTree");
        stacksTree->setSelectionMode(QAbstractItemView::ExtendedSelection);
        stacksTree->setRootIsDecorated(false);
        stacksTree->setUniformRowHeights(true);

        verticalLayout_5->addWidget(stacksTree);

        stacksDock->setWidget(stacksDockContents);
        MainWindow->addDockWidget(Qt::DockWidgetArea::RightDockWidgetArea, stacksDock);
        menuBar = new QMenuBar(MainWindow);
        menuBar->setObjectName("menuBar");
        menuBar->setGeometry(QRect(0, 0, 1332, 29));
        menu_File = new QMenu(menuBar);
        menu_File->setObjectName("menu_File");
        menu_Settings = new QMenu(menuBar);
        menu_Settings->setObjectName("menu_Settings");
        menuFilter = new QMenu(menuBar);
        menuFilter->setObjectName("menuFilter");
        MainWindow->setMenuBar(menuBar);
#if QT_CONFIG(shortcut)
        openFileLabel->setBuddy(openFile);
        compareToLabel->setBuddy(compareTo);
        suppressionsLabel->setBuddy(suppressions);
#endif // QT_CONFIG(shortcut)

        menuBar->addAction(menu_File->menuAction());
        menuBar->addAction(menuFilter->menuAction());
        menuBar->addAction(menu_Settings->menuAction());
        menuFilter->addAction(actionResetFilter);

        retranslateUi(MainWindow);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(tr2i18n("MainWindow", nullptr));
#if QT_CONFIG(tooltip)
        MainWindow->setToolTip(tr2i18n("Path to a file containing leak suppression rules in the LSAN format.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionResetFilter->setText(tr2i18n("Reset Filter", nullptr));
        groupBox->setTitle(tr2i18n("Open Heaptrack Data", nullptr));
#if QT_CONFIG(tooltip)
        openFile->setToolTip(tr2i18n("<qt><p>This field specifies the primary heaptrack data file. These files are called <tt>heaptrack.$APP.$PID.gz</tt> or <tt>heaptrack.$APP.$PID.zst</tt>. You can produce such a file by profiling your application, e.g. via:</p>\n"
"<pre><code>heaptrack &lt;yourapplication&gt; ...</code></pre>\n"
"<p>Or, alternatively, you can attach to a running process via</p>\n"
"<pre><code>heaptrack --pid $(pidof &lt;yourapplication&gt;)</code></pre></qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        openFile->setPlaceholderText(tr2i18n("path/to/heaptrack.$APP.$PID.{gz,zst}", nullptr));
#if QT_CONFIG(tooltip)
        compareTo->setToolTip(tr2i18n("<qt>You can optionally specify a second heaptrack data file to compare to. If set, this file will be used as a base and its cost gets subtracted from the primary data costs.</qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        compareTo->setPlaceholderText(tr2i18n("path/to/heaptrack.$APP.$PID.{gz,zst}", nullptr));
        openFileLabel->setText(tr2i18n("Profile &Data:", nullptr));
        compareToLabel->setText(tr2i18n("Compare to:", nullptr));
#if QT_CONFIG(tooltip)
        suppressionsLabel->setToolTip(tr2i18n("Path to a file containing leak suppression rules in the LSAN format.", nullptr));
#endif // QT_CONFIG(tooltip)
        suppressionsLabel->setText(tr2i18n("Suppressions:", nullptr));
        suppressions->setPlaceholderText(tr2i18n("path/to/lsan_suppressions.txt", nullptr));
#if QT_CONFIG(tooltip)
        topPeakLabel->setToolTip(tr2i18n("List of functions that allocated the most memory at a given time.", nullptr));
#endif // QT_CONFIG(tooltip)
        topPeakLabel->setText(tr2i18n("Peak Contributions", nullptr));
#if QT_CONFIG(tooltip)
        topPeak->setToolTip(tr2i18n("List of functions that allocated the most memory at a given time.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        topLeakedLabel->setToolTip(tr2i18n("List of functions that leak the most memory.", nullptr));
#endif // QT_CONFIG(tooltip)
        topLeakedLabel->setText(tr2i18n("Largest Memory Leaks", nullptr));
#if QT_CONFIG(tooltip)
        topLeaked->setToolTip(tr2i18n("List of functions that leak the most memory.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        topAllocationsLabel->setToolTip(tr2i18n("List of functions that allocate memory most often.", nullptr));
#endif // QT_CONFIG(tooltip)
        topAllocationsLabel->setText(tr2i18n("Most Memory Allocations", nullptr));
#if QT_CONFIG(tooltip)
        topAllocations->setToolTip(tr2i18n("List of functions that allocate memory most often.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        topTemporaryLabel->setToolTip(tr2i18n("List of functions that produced the most temporary memory allocations.", nullptr));
#endif // QT_CONFIG(tooltip)
        topTemporaryLabel->setText(tr2i18n("Most Temporary Allocations", nullptr));
#if QT_CONFIG(tooltip)
        topTemporary->setToolTip(tr2i18n("List of functions that produced the most temporary memory allocations.", nullptr));
#endif // QT_CONFIG(tooltip)
        suppressionBox->setTitle(tr2i18n("Suppressions", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(summaryTab), tr2i18n("Summary", nullptr));
        bottomUpFilterFunction->setPlaceholderText(tr2i18n("filter by function...", nullptr));
        bottomUpFilterFile->setPlaceholderText(tr2i18n("filter by file...", nullptr));
        bottomUpFilterModule->setPlaceholderText(tr2i18n("filter by module...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(bottomUpTab), tr2i18n("Bottom-Up", nullptr));
        callerCalleeFilterFunction->setPlaceholderText(tr2i18n("filter by function...", nullptr));
        callerCalleeFilterModule->setPlaceholderText(tr2i18n("filter by module...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(callerCalleeTab), tr2i18n("Caller / Callee", nullptr));
        topDownFilterFunction->setPlaceholderText(tr2i18n("filter by function...", nullptr));
        topDownFilterFile->setPlaceholderText(tr2i18n("filter by file...", nullptr));
        topDownFilterModule->setPlaceholderText(tr2i18n("filter by module...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(topDownTab), tr2i18n("Top-Down", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(flameGraphTab), tr2i18n("Flame Graph", nullptr));
        stacksDock->setWindowTitle(tr2i18n("S&tacks", nullptr));
        stackSpinnerLabel->setText(tr2i18n("Selected Stack:", nullptr));
        menu_File->setTitle(tr2i18n("&File", nullptr));
        menu_Settings->setTitle(tr2i18n("Setti&ngs", nullptr));
        menuFilter->setTitle(tr2i18n("Filter", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MAINWINDOW_H

